%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

classdef AnalysisFigure < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                 matlab.ui.Figure
        FitMenu                  matlab.ui.container.Menu
        AnalysisLinearMenu       matlab.ui.container.Menu
        AnalysisExponentialMenu  matlab.ui.container.Menu
        AnalysisGaussianMenu     matlab.ui.container.Menu
        AnalysisBoltzmannMenu    matlab.ui.container.Menu
        AnalysisAxes             matlab.ui.control.UIAxes
        ControlsPanel            matlab.ui.container.Panel
        XMapDropDown             matlab.ui.control.DropDown
        XMapLabel                matlab.ui.control.Label
        YMapDropDown             matlab.ui.control.DropDown
        YMapLabel                matlab.ui.control.Label
        FittedSweepSpinner       matlab.ui.control.Spinner
        FittedSweepSpinnerLabel  matlab.ui.control.Label
        UITable                  matlab.ui.control.Table
        TickWarningLabel         matlab.ui.control.Label
    end

    
    properties (Access = public)
        
        % Define variables
        plotNumber = 0;
        plotXData = {[]};
        plotYData = {[]};
        plotDataType = [];
        plotXTick = [];
        plotXTickLabel = {''};
        plotYTick = [];
        plotYTickLabel = {''};
        analysisWindowClosed
        cursorsOk
        
        fitXCoords
        fitYCoords
        fitXResults
        fitYResults
        fitXMappedResults
        fitYMappedResults
        fitXTrasfType
        fitYTrasfType
        expType
        fittedSpinner
        fittedLine
        fittableLinesNum
        fitAxes
        fitPerformed = false;
        fitVariableLength
        fitVariableNumber
        fitMirrorXResults
        fitMirrorYResults
        AnalysisFigureList
        
        % Define transformation constants
        TRANSF_TYPE_LIN = 1
        TRANSF_TYPE_QUAD = 2
        TRANSF_TYPE_LOG = 3
        
        % Defin normalization constants
        NORM_TYPE_NOT = 1
        NORM_TYPE_MAX = 2
        NORM_TYPE_CELL = 3
        
        % Define exponential type constants
        EXP_TYPE_SINGLE = 1
        EXP_TYPE_BI = 2
        
        % Define export structures
        expParam = struct([]);
        gaussParam = struct([]);
        linParam = struct([]);
        boltzParam = struct([]);
        
        % Define figure tag
        tag = 'AnalysisFigure';
        
        xUnitLabel
        yUnitLabel
        normType
        obj
        
    end
    
    methods (Access = public)
        
        function a = getAxes(app)
            % Get AnalysisAxes handle
            a = app.AnalysisAxes;
        end
        
        function assignDataToPlot(app, x, y, type)
            % Associate plotted lines with original data
            assignDataToPlot_m(app, x, y, type);
        end
        
        function savePlotTicks(app)
            % Assign plot ticks to variables
            savePlotTicks_m(app);
        end
        
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Value changed function: XMapDropDown
        function XMapDropDownValueChanged(app, event)
            % Change X axis map (lin,square, sqrt, log)
            dataPropertyName = 'xdata';
            tickPropertyName = 'xtick';
            MapDropDownValueChanged(app, event, dataPropertyName, tickPropertyName);
        end

        % Value changed function: YMapDropDown
        function YMapDropDownValueChanged(app, event)
            % Change Y axis map (lin,square, sqrt, log)
            dataPropertyName = 'ydata';
            tickPropertyName = 'ytick';
            MapDropDownValueChanged(app, event, dataPropertyName, tickPropertyName);
        end

        % Menu selected function: AnalysisLinearMenu
        function AnalysisLinearMenuSelected(app, event)
            % Linear fitting
            app.fitAxes = app.AnalysisAxes;
            AnalysisLinearMenuSelected_m(app);
        end

        % Menu selected function: AnalysisExponentialMenu
        function AnalysisExponentialMenuSelected(app, event)
            % Exponential fitting
            app.fitAxes = app.AnalysisAxes;
            AnalysisExponentialMenuSelected_m(app);
        end

        % Menu selected function: AnalysisGaussianMenu
        function AnalysisGaussianMenuSelected(app, event)
            % Gaussian fitting
            app.fitAxes = app.AnalysisAxes;
            AnalysisGaussianMenuSelected_m(app);
        end

        % Menu selected function: AnalysisBoltzmannMenu
        function AnalysisBoltzmannMenuSelected(app, event)
            % Boltzmann fitting
            app.fitAxes = app.AnalysisAxes;
            AnalysisBoltzmannMenuSelected_m(app);
        end

        % Value changing function: FittedSweepSpinner
        function FittedSweepSpinnerValueChanging(app, event)
            % Plot fitted curve selected by sweep spinner
            app.fitAxes = app.AnalysisAxes;
            originalDataArrays = find(app.plotDataType == 0);
            app.fittableLinesNum = length(originalDataArrays);
            FittedSweepSpinnerValueChanging_m(app, event);
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.Position = [100 100 640 480];
            app.UIFigure.Name = 'UI Figure';

            % Create FitMenu
            app.FitMenu = uimenu(app.UIFigure);
            app.FitMenu.Text = 'Fit';

            % Create AnalysisLinearMenu
            app.AnalysisLinearMenu = uimenu(app.FitMenu);
            app.AnalysisLinearMenu.MenuSelectedFcn = createCallbackFcn(app, @AnalysisLinearMenuSelected, true);
            app.AnalysisLinearMenu.Tooltip = {'m*xdata+b'};
            app.AnalysisLinearMenu.Text = 'Linear';

            % Create AnalysisExponentialMenu
            app.AnalysisExponentialMenu = uimenu(app.FitMenu);
            app.AnalysisExponentialMenu.MenuSelectedFcn = createCallbackFcn(app, @AnalysisExponentialMenuSelected, true);
            app.AnalysisExponentialMenu.Tooltip = {'Single exponential: B+A*exp(-xdata/tau)'; 'Bi-exponential: (B1+A1*exp(-xdata/tau1))+(B2+A2*exp(-xdata/tau2))'};
            app.AnalysisExponentialMenu.Text = 'Exponential';

            % Create AnalysisGaussianMenu
            app.AnalysisGaussianMenu = uimenu(app.FitMenu);
            app.AnalysisGaussianMenu.MenuSelectedFcn = createCallbackFcn(app, @AnalysisGaussianMenuSelected, true);
            app.AnalysisGaussianMenu.Tooltip = {'A/(sqrt(2*pi)*sigma)*exp(-0.5*(((xdata-mu)/sigma).^2))'};
            app.AnalysisGaussianMenu.Text = 'Gaussian';

            % Create AnalysisBoltzmannMenu
            app.AnalysisBoltzmannMenu = uimenu(app.FitMenu);
            app.AnalysisBoltzmannMenu.MenuSelectedFcn = createCallbackFcn(app, @AnalysisBoltzmannMenuSelected, true);
            app.AnalysisBoltzmannMenu.Tooltip = {'bottom + (top-bottom)./(1+exp((vH-xdata)/slope))'};
            app.AnalysisBoltzmannMenu.Text = 'Boltzmann';

            % Create AnalysisAxes
            app.AnalysisAxes = uiaxes(app.UIFigure);
            title(app.AnalysisAxes, 'Title')
            xlabel(app.AnalysisAxes, 'X')
            ylabel(app.AnalysisAxes, 'Y')
            app.AnalysisAxes.Position = [200 14 441 453];

            % Create ControlsPanel
            app.ControlsPanel = uipanel(app.UIFigure);
            app.ControlsPanel.Title = 'Controls';
            app.ControlsPanel.Position = [12 14 189 453];

            % Create XMapDropDown
            app.XMapDropDown = uidropdown(app.ControlsPanel);
            app.XMapDropDown.Items = {'Linear', 'Quadratic', 'Square root', 'Logarithmic'};
            app.XMapDropDown.ValueChangedFcn = createCallbackFcn(app, @XMapDropDownValueChanged, true);
            app.XMapDropDown.Position = [50 408 130 22];
            app.XMapDropDown.Value = 'Linear';

            % Create XMapLabel
            app.XMapLabel = uilabel(app.ControlsPanel);
            app.XMapLabel.Position = [11 408 40 22];
            app.XMapLabel.Text = 'X map';

            % Create YMapDropDown
            app.YMapDropDown = uidropdown(app.ControlsPanel);
            app.YMapDropDown.Items = {'Linear', 'Quadratic', 'Square root', 'Logarithmic'};
            app.YMapDropDown.ValueChangedFcn = createCallbackFcn(app, @YMapDropDownValueChanged, true);
            app.YMapDropDown.Position = [50 379 130 22];
            app.YMapDropDown.Value = 'Linear';

            % Create YMapLabel
            app.YMapLabel = uilabel(app.ControlsPanel);
            app.YMapLabel.Position = [11 379 40 22];
            app.YMapLabel.Text = 'Y map';

            % Create FittedSweepSpinner
            app.FittedSweepSpinner = uispinner(app.ControlsPanel);
            app.FittedSweepSpinner.ValueChangingFcn = createCallbackFcn(app, @FittedSweepSpinnerValueChanging, true);
            app.FittedSweepSpinner.Limits = [1 Inf];
            app.FittedSweepSpinner.ValueDisplayFormat = '%.0f';
            app.FittedSweepSpinner.Enable = 'off';
            app.FittedSweepSpinner.Position = [120 344 60 22];
            app.FittedSweepSpinner.Value = 1;

            % Create FittedSweepSpinnerLabel
            app.FittedSweepSpinnerLabel = uilabel(app.ControlsPanel);
            app.FittedSweepSpinnerLabel.Enable = 'off';
            app.FittedSweepSpinnerLabel.Position = [11 344 85 22];
            app.FittedSweepSpinnerLabel.Text = 'Fitted Sweep #';

            % Create UITable
            app.UITable = uitable(app.ControlsPanel);
            app.UITable.ColumnName = {'Sweep'; 'Parameters'};
            app.UITable.ColumnWidth = {'auto'};
            app.UITable.RowName = {};
            app.UITable.Position = [1 74 187 261];

            % Create TickWarningLabel
            app.TickWarningLabel = uilabel(app.UIFigure);
            app.TickWarningLabel.BackgroundColor = [1 0.502 0.149];
            app.TickWarningLabel.HorizontalAlignment = 'center';
            app.TickWarningLabel.FontWeight = 'bold';
            app.TickWarningLabel.Visible = 'off';
            app.TickWarningLabel.Tooltip = {'Warning: data tips are realiable only in linear scale'; 'Negative values are not transformed'};
            app.TickWarningLabel.Position = [214 457 24 24];
            app.TickWarningLabel.Text = '!';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = AnalysisFigure

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end