%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function BoltzmannFit(app)
% Fit every sweep
for sw = 1:app.fitVariableNumber
  
  % Check if data is a cell type (coming from selection of multiple cursor)
  if iscell(app.fitXCoords)
    x_fit = app.fitXCoords{sw};
    y_fit = app.fitYCoords{sw};
  else
    x_fit = app.fitXCoords(:, sw);
    y_fit = app.fitYCoords(:, sw);
  end
  
  % Check if last value is missing
  nanIdx = find(~isnan(y_fit), 1, 'last');
  x_fit = x_fit(1:nanIdx);
  y_fit = y_fit(1:nanIdx);
  
  % Initialize parameters
  bottom0 = abs(min(y_fit));
  top0 = abs(max(y_fit));
  vH0 = x_fit(round(end/2));
  slope0 = -20*( (top0-bottom0) / abs(y_fit(end)-y_fit(1)) );% to give the sign to the initial slope
  
  %%%%% Fitting (least squares)
  % Initial conditions (defined above)
  x0 = [bottom0,top0,vH0,slope0];
  % fminsearch_error is the least square function; boltzmann_eval is the
  % function containing the model: modify this if you prefer to fit
  % another type of function
  fun = @(x)fminsearch_error(x,x_fit,y_fit,@boltzmann_eval);
  % Retrieve fitted parameters
  boltzmannParam = fminsearch(fun,x0);
  
  % Save the fitted data in a matrix (to be plotted)...
  app.fitXResults(:, sw) = linspace(x_fit(1), x_fit(end), app.fitVariableLength);
  app.fitYResults(:, sw) = boltzmann_eval(boltzmannParam, app.fitXResults(:, sw));
  % ...and fitted parameters in a struct (to be exported)
  app.boltzParam(sw).Bottom = boltzmannParam(1); % save parametrs in a struct
  app.boltzParam(sw).Top = boltzmannParam(2);
  app.boltzParam(sw).VHalf = boltzmannParam(3);
  app.boltzParam(sw).BoltzSlope = boltzmannParam(4);
  
end

% set parameter name and unit of measure in GUIFigure table
if strcmpi(app.tag, 'GUIFigure')
  app.UITable.ColumnName = {'Sweep'; 'VHalf [ms]' ; 'Slope [nA/ms]'};
elseif strcmpi(app.tag, 'AnalysisFigure')
  % set parameter name and unit of measure in AnalysisAxes table
  switch app.normType
    % Unit of measures depends on normalization
    case app.NORM_TYPE_NOT
      app.UITable.ColumnName = {'Sweep'; ['VHalf ' app.xUnitLabel{end} ];[ 'Slope ' app.yUnitLabel{end} '/' app.xUnitLabel{end} ]};
    case app.NORM_TYPE_MAX
      app.UITable.ColumnName = {'Sweep'; ['VHalf ' app.xUnitLabel{end} ];[ 'Slope 1/' app.xUnitLabel{end} ]};
    case app.NORM_TYPE_CELL
      app.UITable.ColumnName = {'Sweep'; ['VHalf ' app.xUnitLabel{end} ];[ 'Slope ' app.yUnitLabel{end} ' /' app.xUnitLabel{end} ]};
  end
end
% fill Table with parameters
app.UITable.Data = [(1:length({app.boltzParam.Bottom}))' cell2mat({app.boltzParam.VHalf}') cell2mat({app.boltzParam.BoltzSlope}') ];
end
