%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function ConductanceGraphMenuSelected_m(app)
%%% Analysis: Conductance graph
InitialAnalysisControls(app);

% Execute Analysis only if cursors are in the right position
if app.cursorsOk
  % Dialog to choose the normalization to be applied to data
  d = dialog('Position',[300 200 250 220],'Name','G/V Data Selection','CloseRequestFcn',{@DataSelectionWindowClosing, app});
  
  txt = uicontrol('Parent',d,...
    'Style','text',...
    'Position',[20 160 210 40],...
    'String','Select in which point calculate the I/V graph');
  
  txt1 = uicontrol('Parent',d,...
    'Style','text',...
    'Position',[0 90 250 50],...
    'String','Select the normalization on Current data');
  
  popupString = {'Tail', 'mean','max','min'};
  popupStringNorm = {'Not normalized','max [0 1]','Cell capacitance'};
  
  popup = uicontrol('Parent',d,...
    'Style','popup',...
    'Position',[75 150 100 25],...
    'String',popupString,...
    'Callback',{@popup_callback,app});
  
  popupNorm = uicontrol('Parent',d,...
    'Style','popup',...
    'Position',[75 70 100 25],...
    'String',popupStringNorm,...
    'Callback',{@popupNorm_callback,app});
  
  btn = uicontrol('Parent',d,...
    'Style','pushbutton',...
    'Position',[89 20 70 25],...
    'String','Select',...
    'Callback',{@select_btn_callback,app});
  
  % Default: 1-> mean
  app.IVpoint = app.POINT_TAIL;
  % Default: 1-> not normalized
  app.normType = app.NORM_TYPE_NOT;
  
  % Wait for d to close before running to completion
  uiwait(d);
  
  % Execute the fitting only if the dialog window hasn't been closed
  if ~app.analysisWindowClosed
    
    % Increment number of executed Analysis
    app.GVindex = app.GVindex+1;
    % Plot on Analysis figure
    f = AnalysisFigure;
    % Keep figure handle in a variable
    app.AnalysisFigureList = [app.AnalysisFigureList, f];
    % Retrieve axes handle
    a = f.getAxes;
    % Assign window closing function to AnalysisFigure
    a.DeleteFcn = {@AnalysisWindowClosing, app, a, f};
    % hold on
    set(a, 'nextplot', 'add');
    
    plot(a,app.Vgraph,app.conductance,'bo')
    % Plot properties
    xlim(a, [min(app.Vgraph)-5 max(app.Vgraph)+5])
    ylim(a, [min(app.conductance)-0.1 max(app.conductance)+0.1])
    grid(a,'on')
    title(a,['Conductance graph ' num2str(app.GVindex)])
    xLab = {'V' ['[' app.voltageAxesLabel ']']};
    xlabel(a,xLab)
    % Select y label according to normalization selected
    switch app.normType
      case app.NORM_TYPE_NOT % not norm
        yLab = {'G ' ['[' app.conductanceAxesLabel ']']};
      case app.NORM_TYPE_MAX % max: [0 1]
        yLab = {'Fractional Activation'};
      case app.NORM_TYPE_CELL % Cell capacitance
        yLab = {'G ' ['[' app.conductanceAxesLabel '/pF]']};
    end
    ylabel(a,yLab)
    
    % Pass the original data to AnalysisAxes
    f.assignDataToPlot(app.Vgraph,app.conductance,0);
    % Pass ticks data to AnalysisAxes
    f.savePlotTicks;
    % Pass labels to AnalysisAxes
    f.xUnitLabel = get(get(a,'xlabel'),'string');
    f.yUnitLabel = get(get(a,'ylabel'),'string');
    % Pass normalization type to AnalysisAxes
    f.normType = app.normType;
    % Save data in a struct to be exported
    tempStruct.conductance = app.conductance;
    tempStruct.voltage = app.Vgraph;
    tempStruct.axes = a;
    tempStruct.title = ['Conductance graph ' num2str(app.GVindex)];
    tempStruct.axesUnits.Conductance = 'uS';
    tempStruct.axesUnits.Voltage = app.voltageAxesLabel;
    % Concatenate executed Analysis in a struct
    app.ConductanceAnalysisResults = [app.ConductanceAnalysisResults; tempStruct];
    
  end
  
end
end

function popup_callback(popup,~,app)
% Set in which point to do the analysis
idx = popup.Value;
popup_items = popup.String;
choice = string(popup_items{idx});

if choice == "Tail"
  app.IVpoint = app.POINT_TAIL;
  
elseif choice == "mean"
  app.IVpoint = app.POINT_MEAN;
  
elseif choice == "max"
  app.IVpoint = app.POINT_MAX;
  
elseif choice == "min"
  app.IVpoint = app.POINT_MIN;
  
end

end

function popupNorm_callback(popup,~,app)
% Set the normalization
idx = popup.Value;
popup_items = popup.String;
choice = string(popup_items{idx});
if startsWith(choice, "Not")
  app.normType = app.NORM_TYPE_NOT;
  
elseif startsWith(choice, "max")
  app.normType = app.NORM_TYPE_MAX;
  
elseif startsWith(choice, "Cell")
  app.normType = app.NORM_TYPE_CELL;
end

end

function select_btn_callback(~,~,app)
% Get the index of the end of the stimulation
volt = app.voltage(app.cursorsTimeVect,:);
curr = app.current(app.cursorsTimeVect,:);
% If there is only one voltage signal and more than 1 current signals then
% we have not several sweeps, but rather several channels, so repeat the
% voltage for each channel
if size(volt, 2) == 1 && size(app.current, 2) > 1
  volt = repmat(volt, 1, size(app.current, 2));
end
voltDiff = diff(volt);
[~,endStimIdx] = max(abs(voltDiff), [], 1);
tailShift = 50;

% Check if the end point of the stimulation for every sweep is in ascending order
for i = 1:size(app.current, 2)
    if i > 1 && i < size(app.current, 2)
        if endStimIdx(i) < endStimIdx(i-1)
            endStimIdx(i) = round((endStimIdx(i-1)+endStimIdx(i+1))/2);
        end
    end
end
% Extract median value to reduce noise
tailStart = endStimIdx+tailShift;
tailInt = 100;
for idx = size(app.current,2):-1:1
    app.Vgraph(idx) = mean(volt(1:endStimIdx(idx),idx))';
end

switch app.IVpoint
  case app.POINT_TAIL % tail currents
    
    for idx = size(app.current,2):-1:1
      meanTail(idx) = mean(curr(tailStart(idx):tailStart(idx)+tailInt,idx));
    end
    app.conductance = abs(meanTail);
        
  case app.POINT_MEAN % mean
    % One mean value for every sweep (between cursors)
    app.IGgraph = mean(app.current(app.cursorsTimeVect,:))';
    % Resting potential (assumed in mV)
    Ek = app.RestingPotentialField.Value;
    % Conductance
    app.conductance = app.IGgraph./(app.Vgraph-Ek);
    
  case app.POINT_MAX % max
    % One max value for every sweep (between cursors)
    app.IGgraph = max(app.current(app.cursorsTimeVect,:))';
    % Resting potential (assumed in mV)
    Ek = app.RestingPotentialField.Value;
    % Conductance
    app.conductance = app.IGgraph./(app.Vgraph-Ek);
    
  case app.POINT_MIN % min
    % One min value for every sweep (between cursors)
    app.IGgraph = min(app.current(app.cursorsTimeVect,:))';
    % Resting potential (assumed in mV)
    Ek = app.RestingPotentialField.Value;
    % Conductance
    app.conductance = app.IGgraph./(app.Vgraph-Ek);
    
end

% Apply normalization
switch app.normType
  case app.NORM_TYPE_NOT % Not normalized
    % nothing to do
  case app.NORM_TYPE_MAX % max [0 1]
    app.conductance = (app.conductance-min(app.conductance))./(max(app.conductance)-min(app.conductance));
  case app.NORM_TYPE_CELL % Cell capacitance
    % Check if Cell capacitance value was inserted, otherwise use default
    % value = 1
    if isempty(app.cellCapacitance) || app.cellCapacitance == 0 || app.cellCapacitance == 1
      app.cellCapacitance = 1;
    end
    app.conductance = app.conductance/app.cellCapacitance;
end
% Close dialog
delete(gcf)
end
