%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

classdef EDAmat < matlab.apps.AppBase
  
  % Properties that correspond to app components
  properties (Access = public)
    EDAmatUIFigure              matlab.ui.Figure
    FileMenu                    matlab.ui.container.Menu
    LoadFileMenu                matlab.ui.container.Menu
    ExporttomatMenu             matlab.ui.container.Menu
    AnalyzeMenu                 matlab.ui.container.Menu
    IVgraphMenu                 matlab.ui.container.Menu
    ConductancegraphMenu        matlab.ui.container.Menu
    HistogramMenu               matlab.ui.container.Menu
    PSDMenu                     matlab.ui.container.Menu
    DependenceAnalysisMenu      matlab.ui.container.Menu
    FitMenu                     matlab.ui.container.Menu
    ExponentialMenu             matlab.ui.container.Menu
    GaussianMenu                matlab.ui.container.Menu
    LinearMenu                  matlab.ui.container.Menu
    BoltzmannMenu               matlab.ui.container.Menu
    About                       matlab.ui.container.Menu
    DataInfoMenu                matlab.ui.container.Menu
    CurrentAxes                 matlab.ui.control.UIAxes
    VoltageAxes                 matlab.ui.control.UIAxes
    CursorNumberSpinner         matlab.ui.control.Spinner
    CursorDropDown              matlab.ui.control.DropDown
    x_cursorSpinner             matlab.ui.control.Spinner
    FittedSweepSpinner          matlab.ui.control.Spinner
    FittedSweepSpinnerLabel     matlab.ui.control.Label
    CursorSpinnerLabel          matlab.ui.control.Label
    RestingPotentialField       matlab.ui.control.NumericEditField
    RestingPotentialmVLabel     matlab.ui.control.Label
    WarningLabel                matlab.ui.control.Label
    ExperimentalconditionField  matlab.ui.control.EditField
    ExperimentalconditionLabel  matlab.ui.control.Label
    CellCapacitanceField        matlab.ui.control.NumericEditField
    CellCapacitancepFLabel      matlab.ui.control.Label
    CapacitanceWarningLabel     matlab.ui.control.Label
    UITable                     matlab.ui.control.Table
  end
  
  
  properties (Access = public)
    
    % Define variables
    dataTimeVect
    infoAbfLoad
    sweepLength
    sweepsNumber
    channelsNumber
    samplingTime
    samplingFrequency
    current
    voltage
    fileName
    cursorsLine
    curLabel
    selectedSweepNum
    restingPotential
    rawExpText
    IVpoint
    cursorsTimeVect
    IGgraph
    Vgraph
    conductance
    cellCapacitance
    psdMod
    posTrasfError
    analysisWindowClosed
    cursorsOk
    
    % Define axes unit labels
    currentAxesLabel
    voltageAxesLabel
    timeAxesLabel
    freqAxesLAbel
    conductanceAxesLabel
    
    % Allocate analysis structs
    IVAnalysisResults = struct([]);
    ConductanceAnalysisResults = struct([]);
    HistogramAnalysisResults = struct([]);
    PSDAnalysisResults = struct([]);
    DependenceAnalysisResults = struct([]);
    
    % Define fitting variables
    fitDataType
    fitDataIdx
    fitXCoords
    fitYCoords
    fitXResults
    fitYResults
    fitXMappedResults
    fitYMappedResults
    fittedSpinner
    fittedLine
    fittableLinesNum
    fitAxes
    fitVariableLength
    fitVariableNumber
    fitMirrorXResults
    fitMirrorYResults
    
    % Allocate fitted parameters struct
    expParam = struct([]);
    gaussParam = struct([]);
    linParam = struct([]);
    boltzParam = struct([]);
    analysisFittedParameters = struct([]);
    
    fitXTrasfType
    fitYTrasfType
    trasfTypeDialog
    normType
    expType
    
    % Define transformation constants
    TRANSF_TYPE_LIN = 1
    TRANSF_TYPE_QUAD = 2
    TRANSF_TYPE_LOG = 3
    
    % Define IV and GV graph point constants
    POINT_TAIL = 1
    POINT_MEAN = 2
    POINT_MAX = 3
    POINT_MIN = 4
    
    % Define normalization constants
    NORM_TYPE_NOT = 1
    NORM_TYPE_MAX = 2
    NORM_TYPE_CELL = 3
    
    % Define exponential tye constants
    EXP_TYPE_SINGLE = 1
    EXP_TYPE_BI = 2
    
    % Define analysis indices
    IVindex
    GVindex
    HISTindex
    PSDindex
    DepIndex
    
    AnalysisFigureList = [];
    
    % Define figure tag
    tag = 'GUIFigure';
    
    % Define dependence graph variables
    xUnitLabel
    yUnitLabel
    Ipoint
    Vpoint
    xDep
    yDep
    IdepGraph
    VdepGraph
    xDepType
    yDepType
    xLab
    yLab
    
    % Define dependence graph constants
    DEP_TYPE_I = 1;
    DEP_TYPE_V = 2;
    DEP_TYPE_ExpAmp = 3;
    DEP_TYPE_TAU = 4;
    DEP_TYPE_FULL = 5;
    DEP_TYPE_GaussAmp = 6;
    DEP_TYPE_MEAN = 7;
    DEP_TYPE_STD = 8;
    DEP_TYPE_SLOPE = 9;
    DEP_TYPE_INTERC = 10;
    DEP_TYPE_BOTTOM = 11;
    DEP_TYPE_TOP = 12;
    DEP_TYPE_VHALF = 13;
    DEP_TYPE_BoltzSlope = 14;
    
  end
  
  
  % Callbacks that handle component events
  methods (Access = private)
    
    % Code that executes after component creation
    function startupFcn(app)
      % Create UIFigure mouse actions callback
      set(app.EDAmatUIFigure, ...
        'WindowButtonDownFcn',   @mouseDownCallback, ...
        'WindowButtonUpFcn',     @mouseUpCallback,   ...
        'WindowButtonMotionFcn', @mouseMotionCallback);
    end
    
    % Menu selected function: LoadFileMenu
    function LoadFileMenuSelected(app, event)
      % Load one or more files
      LoadFileMenuSelected_m(app);
    end
    
    % Value changing function: CursorNumberSpinner
    function CursorNumberSpinnerValueChanging(app, event)
      % Create one or more cursors
      CursorNumberSpinnerValueChanging_m(app, event);
    end
    
    % Value changing function: x_cursorSpinner
    function x_cursorSpinnerValueChanging(app, event)
      % Move a cursor
      x_cursorSpinnerValueChanging_m(app, event);
    end
    
    % Value changed function: CursorDropDown
    function CursorDropDownValueChanged(app, event)
      % Select one of the existing cursors
      CursorDropDownValueChanged_m(app);
    end
    
    % Menu selected function: ExporttomatMenu
    function ExporttomatMenuSelected(app, event)
      % Export data, information, analysis and fittings
      ExporttomatMenuSelected_m(app);
    end
    
    % Menu selected function: IVgraphMenu
    function IVgraphMenuSelected(app, event)
      % IV Analysis
      app.fitAxes = app.CurrentAxes;
      IVgraphMenuSelected_m(app);
    end
    
    % Menu selected function: DataInfoMenu
    function DataInfoMenuSelected(app, event)
      % Display data information
      DataInfoMenuSelected_m(app);
    end
    
    % Menu selected function: ExponentialMenu
    function ExponentialMenuSelected(app, event)
      % Exponential fitting
      app.fitAxes = app.CurrentAxes;
      ExponentialMenuSelected_m(app);
    end
    
    % Value changing function: FittedSweepSpinner
    function FittedSweepSpinnerValueChanging(app, event)
      % Select fit of a specific sweep to be plotted
      app.fitAxes = app.CurrentAxes;
      FittedSweepSpinnerValueChanging_m(app, event);
    end
    
    % Menu selected function: HistogramMenu
    function HistogramMenuSelected(app, event)
      % Histogram analysis
      app.fitAxes = app.CurrentAxes;
      HistogramMenuSelected_m(app);
    end
    
    % Menu selected function: GaussianMenu
    function GaussianMenuSelected(app, event)
      % Gaussian fitting
      app.fitAxes = app.CurrentAxes;
      GaussianMenuSelected_m(app);
    end
    
    % Menu selected function: LinearMenu
    function LinearMenuSelected(app, event)
      % Linear fitting
      app.fitAxes = app.CurrentAxes;
      LinearMenuSelected_m(app);
    end
    
    % Menu selected function: BoltzmannMenu
    function BoltzmannMenuSelected(app, event)
      % Boltzmann fitting
      app.fitAxes = app.CurrentAxes;
      BoltzmannMenuSelected_m(app);
    end
    
    % Menu selected function: ConductancegraphMenu
    function ConductancegraphMenuSelected(app, event)
      % Conductance analysis
      app.fitAxes = app.CurrentAxes;
      ConductanceGraphMenuSelected_m(app);
    end
    
    % Menu selected function: PSDMenu
    function PSDMenuSelected(app, event)
      % Power spectral density analysis
      app.fitAxes = app.CurrentAxes;
      PSDMenuSelected_m(app);
    end
    
    % Value changed function: RestingPotentialField
    function RestingPotentialFieldValueChanged(app, event)
      % Handle resting potential warning label visibility
      RestingPotentialFieldValueChanged_m(app);
    end
    
    % Value changed function: CellCapacitanceField
    function CellCapacitanceFieldValueChanged(app, event)
      % Handle cell capacitance warning label visibility
      CellCapacitanceFieldValueChanged_m(app);
    end
    
    % Menu selected function: DependenceAnalysisMenu
    function DependenceAnalysisMenuSelected(app, event)
      % Dependence analysis
      DependenceAnalysisMenuSelected_m(app);
    end
  end
  
  % Component initialization
  methods (Access = private)
    
    % Create UIFigure and components
    function createComponents(app)
      
      % Create EDAmatUIFigure and hide until all components are created
      app.EDAmatUIFigure = uifigure('Visible', 'off');
      app.EDAmatUIFigure.Position = [100 100 670 480];
      app.EDAmatUIFigure.Name = 'EDAmat';
      
      % Create FileMenu
      app.FileMenu = uimenu(app.EDAmatUIFigure);
      app.FileMenu.Text = 'File';
      
      % Create LoadFileMenu
      app.LoadFileMenu = uimenu(app.FileMenu);
      app.LoadFileMenu.MenuSelectedFcn = createCallbackFcn(app, @LoadFileMenuSelected, true);
      app.LoadFileMenu.Text = 'Load File...';
      
      % Create ExporttomatMenu
      app.ExporttomatMenu = uimenu(app.FileMenu);
      app.ExporttomatMenu.MenuSelectedFcn = createCallbackFcn(app, @ExporttomatMenuSelected, true);
      app.ExporttomatMenu.Enable = 'off';
      app.ExporttomatMenu.Text = 'Export to .mat...';
      
      % Create AnalyzeMenu
      app.AnalyzeMenu = uimenu(app.EDAmatUIFigure);
      app.AnalyzeMenu.Tooltip = {'If more than 2 cursors are selected'; 'analysis is executed between cursors 1 and 2'};
      app.AnalyzeMenu.Text = 'Analyze';
      
      % Create IVgraphMenu
      app.IVgraphMenu = uimenu(app.AnalyzeMenu);
      app.IVgraphMenu.MenuSelectedFcn = createCallbackFcn(app, @IVgraphMenuSelected, true);
      app.IVgraphMenu.Tooltip = {''};
      app.IVgraphMenu.Enable = 'off';
      app.IVgraphMenu.Text = 'I/V graph';
      
      % Create ConductancegraphMenu
      app.ConductancegraphMenu = uimenu(app.AnalyzeMenu);
      app.ConductancegraphMenu.MenuSelectedFcn = createCallbackFcn(app, @ConductancegraphMenuSelected, true);
      app.ConductancegraphMenu.Tooltip = {''};
      app.ConductancegraphMenu.Enable = 'off';
      app.ConductancegraphMenu.Text = 'Conductance graph';
      
      % Create HistogramMenu
      app.HistogramMenu = uimenu(app.AnalyzeMenu);
      app.HistogramMenu.MenuSelectedFcn = createCallbackFcn(app, @HistogramMenuSelected, true);
      app.HistogramMenu.Tooltip = {''};
      app.HistogramMenu.Enable = 'off';
      app.HistogramMenu.Text = 'Histogram';
      
      % Create PSDMenu
      app.PSDMenu = uimenu(app.AnalyzeMenu);
      app.PSDMenu.MenuSelectedFcn = createCallbackFcn(app, @PSDMenuSelected, true);
      app.PSDMenu.Tooltip = {''};
      app.PSDMenu.Enable = 'off';
      app.PSDMenu.Text = 'Power Spectral Density';
      
      % Create DependenceAnalysisMenu
      app.DependenceAnalysisMenu = uimenu(app.AnalyzeMenu);
      app.DependenceAnalysisMenu.MenuSelectedFcn = createCallbackFcn(app, @DependenceAnalysisMenuSelected, true);
      app.DependenceAnalysisMenu.Tooltip = {''};
      app.DependenceAnalysisMenu.Enable = 'off';
      app.DependenceAnalysisMenu.Text = 'Dependence Analysis';
      
      % Create FitMenu
      app.FitMenu = uimenu(app.EDAmatUIFigure);
      app.FitMenu.Tooltip = {'If more than 2 cursors are selected cursor 1 is where the fit starts'; ' and the other cursors are where the fit ends (so that cursNum-1 sweeps are fitted)'};
      app.FitMenu.Text = 'Fit';
      
      % Create ExponentialMenu
      app.ExponentialMenu = uimenu(app.FitMenu);
      app.ExponentialMenu.MenuSelectedFcn = createCallbackFcn(app, @ExponentialMenuSelected, true);
      app.ExponentialMenu.Tooltip = {'Single exponential: B+A*exp(-xdata/tau)'; 'Bi-exponential:       (B1+A1*exp(-xdata/tau1))+(B2+A2*exp(-xdata/tau2))'; ''};
      app.ExponentialMenu.Enable = 'off';
      app.ExponentialMenu.Text = 'Exponential';
      
      % Create GaussianMenu
      app.GaussianMenu = uimenu(app.FitMenu);
      app.GaussianMenu.MenuSelectedFcn = createCallbackFcn(app, @GaussianMenuSelected, true);
      app.GaussianMenu.Tooltip = {'A/(sqrt(2*pi)*sigma)*exp(-0.5*(((xdata-mu)/sigma).^2))'};
      app.GaussianMenu.Enable = 'off';
      app.GaussianMenu.Text = 'Gaussian';
      
      % Create LinearMenu
      app.LinearMenu = uimenu(app.FitMenu);
      app.LinearMenu.MenuSelectedFcn = createCallbackFcn(app, @LinearMenuSelected, true);
      app.LinearMenu.Tooltip = {'m*xdata+b'};
      app.LinearMenu.Enable = 'off';
      app.LinearMenu.Text = 'Linear';
      
      % Create BoltzmannMenu
      app.BoltzmannMenu = uimenu(app.FitMenu);
      app.BoltzmannMenu.MenuSelectedFcn = createCallbackFcn(app, @BoltzmannMenuSelected, true);
      app.BoltzmannMenu.Tooltip = {'bottom + (top-bottom)./(1+exp((vH-xdata)/slope))'};
      app.BoltzmannMenu.Enable = 'off';
      app.BoltzmannMenu.Text = 'Boltzmann';
      
      % Create About
      app.About = uimenu(app.EDAmatUIFigure);
      app.About.Text = '?';
      
      % Create DataInfoMenu
      app.DataInfoMenu = uimenu(app.About);
      app.DataInfoMenu.MenuSelectedFcn = createCallbackFcn(app, @DataInfoMenuSelected, true);
      app.DataInfoMenu.Enable = 'off';
      app.DataInfoMenu.Text = 'Data Info';
      
      % Create CurrentAxes
      app.CurrentAxes = uiaxes(app.EDAmatUIFigure);
      title(app.CurrentAxes, 'Current')
      xlabel(app.CurrentAxes, 't')
      ylabel(app.CurrentAxes, 'A')
      app.CurrentAxes.Position = [171 235 449 235];
      
      % Create VoltageAxes
      app.VoltageAxes = uiaxes(app.EDAmatUIFigure);
      title(app.VoltageAxes, 'Voltage')
      xlabel(app.VoltageAxes, 't')
      ylabel(app.VoltageAxes, 'V')
      app.VoltageAxes.Position = [171 17 449 206];
      
      % Create CursorNumberSpinner
      app.CursorNumberSpinner = uispinner(app.EDAmatUIFigure);
      app.CursorNumberSpinner.ValueChangingFcn = createCallbackFcn(app, @CursorNumberSpinnerValueChanging, true);
      app.CursorNumberSpinner.Limits = [0 12];
      app.CursorNumberSpinner.ValueDisplayFormat = '%1.0d';
      app.CursorNumberSpinner.Enable = 'off';
      app.CursorNumberSpinner.Position = [86 435 75 22];
      
      % Create CursorDropDown
      app.CursorDropDown = uidropdown(app.EDAmatUIFigure);
      app.CursorDropDown.Items = {};
      app.CursorDropDown.ValueChangedFcn = createCallbackFcn(app, @CursorDropDownValueChanged, true);
      app.CursorDropDown.Enable = 'off';
      app.CursorDropDown.Position = [16 403 145 22];
      app.CursorDropDown.Value = {};
      
      % Create x_cursorSpinner
      app.x_cursorSpinner = uispinner(app.EDAmatUIFigure);
      app.x_cursorSpinner.ValueChangingFcn = createCallbackFcn(app, @x_cursorSpinnerValueChanging, true);
      app.x_cursorSpinner.Limits = [0 Inf];
      app.x_cursorSpinner.ValueDisplayFormat = '%3.5g';
      app.x_cursorSpinner.Enable = 'off';
      app.x_cursorSpinner.Tooltip = {'You can also move the cursors with your mouse'};
      app.x_cursorSpinner.Position = [16 362 145 22];
      
      % Create FittedSweepSpinner
      app.FittedSweepSpinner = uispinner(app.EDAmatUIFigure);
      app.FittedSweepSpinner.ValueChangingFcn = createCallbackFcn(app, @FittedSweepSpinnerValueChanging, true);
      app.FittedSweepSpinner.Limits = [1 Inf];
      app.FittedSweepSpinner.ValueDisplayFormat = '%.0f';
      app.FittedSweepSpinner.Enable = 'off';
      app.FittedSweepSpinner.Position = [106 328 55 22];
      app.FittedSweepSpinner.Value = 1;
      
      % Create FittedSweepSpinnerLabel
      app.FittedSweepSpinnerLabel = uilabel(app.EDAmatUIFigure);
      app.FittedSweepSpinnerLabel.HorizontalAlignment = 'right';
      app.FittedSweepSpinnerLabel.Enable = 'off';
      app.FittedSweepSpinnerLabel.Position = [16 325 86 28];
      app.FittedSweepSpinnerLabel.Text = 'Fitted Sweep #';
      
      % Create CursorSpinnerLabel
      app.CursorSpinnerLabel = uilabel(app.EDAmatUIFigure);
      app.CursorSpinnerLabel.HorizontalAlignment = 'right';
      app.CursorSpinnerLabel.Enable = 'off';
      app.CursorSpinnerLabel.Position = [9 435 52 22];
      app.CursorSpinnerLabel.Text = 'Cursor #';
      
      % Create RestingPotentialField
      app.RestingPotentialField = uieditfield(app.EDAmatUIFigure, 'numeric');
      app.RestingPotentialField.ValueChangedFcn = createCallbackFcn(app, @RestingPotentialFieldValueChanged, true);
      app.RestingPotentialField.Enable = 'off';
      app.RestingPotentialField.Position = [106 290 55 22];
      
      % Create RestingPotentialmVLabel
      app.RestingPotentialmVLabel = uilabel(app.EDAmatUIFigure);
      app.RestingPotentialmVLabel.Enable = 'off';
      app.RestingPotentialmVLabel.Position = [16 287 86 29];
      app.RestingPotentialmVLabel.Text = {'Resting '; 'Potential [mV]'};
      
      % Create WarningLabel
      app.WarningLabel = uilabel(app.EDAmatUIFigure);
      app.WarningLabel.BackgroundColor = [1 0.502 0.149];
      app.WarningLabel.HorizontalAlignment = 'center';
      app.WarningLabel.FontWeight = 'bold';
      app.WarningLabel.Tooltip = {'Please insert a Resting Potential value'; 'Default value: 0'; ''};
      app.WarningLabel.Position = [160 290 25 22];
      app.WarningLabel.Text = '!';
      
      % Create ExperimentalconditionField
      app.ExperimentalconditionField = uieditfield(app.EDAmatUIFigure, 'text');
      app.ExperimentalconditionField.Enable = 'off';
      app.ExperimentalconditionField.Tooltip = {'e.g. : Wild Type'};
      app.ExperimentalconditionField.Position = [16 214 145 22];
      
      % Create ExperimentalconditionLabel
      app.ExperimentalconditionLabel = uilabel(app.EDAmatUIFigure);
      app.ExperimentalconditionLabel.Enable = 'off';
      app.ExperimentalconditionLabel.Position = [16 235 145 22];
      app.ExperimentalconditionLabel.Text = 'Experimental condition';
      
      % Create CellCapacitanceField
      app.CellCapacitanceField = uieditfield(app.EDAmatUIFigure, 'numeric');
      app.CellCapacitanceField.Limits = [1 Inf];
      app.CellCapacitanceField.ValueChangedFcn = createCallbackFcn(app, @CellCapacitanceFieldValueChanged, true);
      app.CellCapacitanceField.Enable = 'off';
      app.CellCapacitanceField.Position = [106 177 55 22];
      app.CellCapacitanceField.Value = 1;
      
      % Create CellCapacitancepFLabel
      app.CellCapacitancepFLabel = uilabel(app.EDAmatUIFigure);
      app.CellCapacitancepFLabel.Enable = 'off';
      app.CellCapacitancepFLabel.Position = [16 174 93 29];
      app.CellCapacitancepFLabel.Text = {'Cell capacitance'; '[pF]'};
      
      % Create CapacitanceWarningLabel
      app.CapacitanceWarningLabel = uilabel(app.EDAmatUIFigure);
      app.CapacitanceWarningLabel.BackgroundColor = [1 0.502 0.149];
      app.CapacitanceWarningLabel.HorizontalAlignment = 'center';
      app.CapacitanceWarningLabel.FontWeight = 'bold';
      app.CapacitanceWarningLabel.Tooltip = {'Please insert a cell capacitance value'; 'Default value: 1'};
      app.CapacitanceWarningLabel.Position = [160 177 25 22];
      app.CapacitanceWarningLabel.Text = '!';
      
      % Create UITable
      app.UITable = uitable(app.EDAmatUIFigure);
      app.UITable.ColumnName = {'Sweep'; 'Parameters'};
      app.UITable.RowName = {};
      app.UITable.Enable = 'off';
      app.UITable.Position = [16 17 145 150];
      
      % Show the figure after all components are created
      app.EDAmatUIFigure.Visible = 'on';
    end
  end
  
  % App creation and deletion
  methods (Access = public)
    
    % Construct app
    function app = EDAmat
      
      % Create UIFigure and components
      createComponents(app)
      
      % Register the app with App Designer
      registerApp(app, app.EDAmatUIFigure)
      
      % Execute the startup function
      runStartupFcn(app, @startupFcn)
      
      if nargout == 0
        clear app
      end
    end
    
    % Code that executes before app deletion
    function delete(app)
      
      % Delete UIFigure when app is deleted
      delete(app.EDAmatUIFigure)
    end
  end
end