%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function ExponentialFit(app)
% Fit every sweep
for sw = 1:app.fitVariableNumber
  
  % Check if data is a cell type (coming from selection of multiple cursor)
  if iscell(app.fitXCoords)
    x_fit = app.fitXCoords{sw};
    y_fit = app.fitYCoords{sw};
  else
    x_fit = app.fitXCoords(:, sw);
    y_fit = app.fitYCoords(:, sw);
  end
  
  % Check if last value is missing
  nanIdx = find(~isnan(y_fit), 1, 'last');
  x_fit = x_fit(1:nanIdx);
  y_fit = y_fit(1:nanIdx);
  
  % Fit from absolute max value (to also get negative peaks)
  [c_max,c_maxIdx] = max(abs(y_fit-y_fit(end)));
  x_fit = x_fit(c_maxIdx:end);
  y_fit = y_fit(c_maxIdx:end);
  
  % to export the right x in fitXResults below
  x_off = x_fit(1);
  
  % First x value must be 0
  x_fit = x_fit-x_fit(1);
  
  % Initialize parameters
  reg = y_fit(end);
  % Time constant extimation: tau is the time a system takes to get to
  % 33% of peak value after a stimulus
  t66 = find(abs(y_fit-reg) < c_max*(1/3), 1, 'first');
  if isempty(t66)
    t66 = length(x_fit);
  end
  tau0 = x_fit(t66);
  
  if app.expType == 1 % Single ezponential
    
    %%%%% Fitting (least squares)
    % Initial conditions (defined above)
    x0 = [c_max-reg,tau0,reg];
    % fminsearch_error is the least square function; exp_eval is the
    % function containing the model: modify this if you prefer to fit
    % another type of function
    fun = @(x)fminsearch_error(x,x_fit,y_fit,@exp_eval);
    % Retrieve parameters
    expParam = fminsearch(fun,x0);
    
    % Save the fitted data in a matrix (to be plotted)...
    app.fitXResults(:, sw) = linspace(x_fit(1), x_fit(end), app.fitVariableLength);
    app.fitYResults(:, sw) = exp_eval(expParam,app.fitXResults(:, sw));
    app.fitXResults(:, sw) = app.fitXResults(:, sw)+x_off;
    
    % ...and fitted parameters in a struct (to be exported)
    app.expParam(sw).ExpAmplitude = expParam(1);
    app.expParam(sw).TimeConstant = expParam(2);
    app.expParam(sw).FullValue = expParam(3);
    
  elseif app.expType == 2 % Bi-exponential
    
    %%%%% Fitting (least squares)
    % Initial conditions (defined above)
    x0 = [c_max-reg,tau0,reg,c_max-reg,tau0,reg];
    % fminsearch_error is the least square function; bi_exp_eval is the
    % function containing the model: modify this if you prefer to fit
    % another type of function
    fun = @(x)fminsearch_error(x,x_fit,y_fit,@bi_exp_eval);
    % Retrieve parameters
    biExpParam = fminsearch(fun,x0);
    
    % Save the fitted data in a matrix (to be plotted)...
    app.fitXResults(:, sw) = linspace(x_fit(1), x_fit(end), app.fitVariableLength);
    app.fitYResults(:, sw) = bi_exp_eval(biExpParam,app.fitXResults(:, sw));
    app.fitXResults(:, sw) = app.fitXResults(:, sw)+x_off;
    
    % ...and fitted parameters in a struct (to be exported)
    app.expParam(sw).ExpAmplitude = [biExpParam(1) biExpParam(4)]; % save parameters in a struct
    app.expParam(sw).TimeConstant = [biExpParam(2) biExpParam(5)];
    app.expParam(sw).FullValue    = [biExpParam(3) biExpParam(6)];
    
  end
  
end

% set parameter name and unit of measure in GUIFigure table
if strcmpi(app.tag, 'GUIFigure')
  app.UITable.ColumnName = {'Sweep';'Time constant [ms]';'Amplitude'};
elseif strcmpi(app.tag, 'AnalysisFigure')
  % set parameter name and unit of measure in AnalysisAxes table
  app.UITable.ColumnName = {'Sweep';['tau ' app.xUnitLabel{end}];'Amplitude'};
end

% Change Table structure if the fit is bi-exponential
if app.expType == 2
  app.UITable.ColumnName = {'Sweep';'Time constant 1 [ms]'; 'Time constant 2 [ms]';'Amplitude 1'; 'Amplitude 2' };
end

% fill Table with parameters
app.UITable.Data = [(1:length({app.expParam.ExpAmplitude}))' cell2mat({app.expParam.TimeConstant}') cell2mat({app.expParam.ExpAmplitude}')];
end
