%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function ExporttomatMenuSelected_m(app)
%%% Create struct to be exported

% Create name of .mat file to be saved
if iscell(app.fileName)
  % if more than one file has been loaded, change the name to finish in
  % "avg", since all the analysis are done on the average of loaded raw
  % data
  nameOfFile = app.fileName{1};
  nameOfFile = [nameOfFile(1:end-4) '_avg'];
else
  nameOfFile = app.fileName(1:end-4);
end
Data.FileName = [nameOfFile '.mat'];
% Time vector
Data.Time = app.dataTimeVect;
% Current data
Data.Current = app.current;
% Voltage data
Data.Voltage = app.voltage;
% Sampling frequency
Data.SamplingFrequency = app.samplingFrequency;
% Axes Units
Data.AxesUnits.Current = app.currentAxesLabel;
Data.AxesUnits.Voltage = app.voltageAxesLabel;
Data.AxesUnits.Time = app.timeAxesLabel;
Data.AxesUnits.SamplingFrequency = app.freqAxesLAbel;
Data.AxesUnits.Conductance = app.conductanceAxesLabel;
% Experimental condition (e.g. wild type)
Data.ExpCondition = app.ExperimentalconditionField.Value;

%%% Exporting fittings
% Function that gets fitting parameters from AnalysiFigure
saveParamToExport(app);
% Store parameters of fittings done on AnalysisFigure
Data.AnalysisFittedParameters = app.analysisFittedParameters;
% Store parameters of fittings done on CurrentAxes
Data.FittedParameters.Exponential = app.expParam;
Data.FittedParameters.Gaussian = app.gaussParam;
Data.FittedParameters.Linear = app.linParam;
Data.FittedParameters.Boltzmann = app.boltzParam;

%%% Exporting Analysis

% Define temporary analysis struct
IVAnalysisExport = struct([]);
realIndex = 0;
for idx = 1:length(app.IVAnalysisResults)
  % Don't store cleared Analysis
  if ~isempty(app.IVAnalysisResults(idx).current)
    % Avoid empty struct array rows (indexing with idx instead of
    % realIndex, if an analysis is cleared, makes that an empty struct
    % is stored)
    realIndex = realIndex+1;
    % Store analysis variables
    IVAnalysisExport(realIndex).current = app.IVAnalysisResults(idx).current;
    IVAnalysisExport(realIndex).voltage = app.IVAnalysisResults(idx).voltage;
    IVAnalysisExport(realIndex).title = app.IVAnalysisResults(idx).title;
    IVAnalysisExport(realIndex).axesUnits = app.IVAnalysisResults(idx).axesUnits;
  end
end
% Store temporary analysis struct in Data struct
Data.Analysis.IVGraph = IVAnalysisExport;

% Define temporary analysis struct
ConductanceAnalysisExport = struct([]);
realIndex = 0;
for idx = 1:length(app.ConductanceAnalysisResults)
  % Don't store cleared Analysis
  if ~isempty(app.ConductanceAnalysisResults(idx).conductance)
    % Avoid empty struct array rows (indexing with idx instead of
    % realIndex, if an analysis is cleared, makes that an empty struct
    % is stored)
    realIndex = realIndex+1;
    % Store analysis variables
    ConductanceAnalysisExport(realIndex).conductance = app.ConductanceAnalysisResults(idx).conductance;
    ConductanceAnalysisExport(realIndex).voltage = app.ConductanceAnalysisResults(idx).voltage;
    ConductanceAnalysisExport(realIndex).title = app.ConductanceAnalysisResults(idx).title;
    ConductanceAnalysisExport(realIndex).axesUnits = app.ConductanceAnalysisResults(idx).axesUnits;
  end
end
% Store temporary analysis struct in Data struct
Data.Analysis.ConductanceGraph = ConductanceAnalysisExport;

% Define temporary analysis struct
HistogramAnalysisExport = struct([]);
realIndex = 0;
for idx = 1:length(app.HistogramAnalysisResults)
  % Don't store cleared Analysis
  if ~isempty(app.HistogramAnalysisResults(idx).count)
    % Avoid empty struct array rows (indexing with idx instead of
    % realIndex, if an analysis is cleared, makes that an empty struct
    % is stored)
    realIndex = realIndex+1;
    % Store analysis variables
    HistogramAnalysisExport(realIndex).count = app.HistogramAnalysisResults(idx).count;
    HistogramAnalysisExport(realIndex).binCenters = app.HistogramAnalysisResults(idx).binCenters;
    HistogramAnalysisExport(realIndex).title = app.HistogramAnalysisResults(idx).title;
    HistogramAnalysisExport(realIndex).axesUnits = app.HistogramAnalysisResults(idx).axesUnits;
  end
end
% Store temporary analysis struct in Data struct
Data.Analysis.Histogram = HistogramAnalysisExport;

% Define temporary analysis struct
PSDAnalysisExport = struct([]);
realIndex = 0;
for idx = 1:length(app.PSDAnalysisResults)
  % Don't store cleared Analysis
  if ~isempty(app.PSDAnalysisResults(idx).currentPSD)
    % Avoid empty struct array rows (indexing with idx instead of
    % realIndex, if an analysis is cleared, makes that an empty struct
    % is stored)
    realIndex = realIndex+1;
    % Store analysis variables
    PSDAnalysisExport(realIndex).currentPSD = app.PSDAnalysisResults(idx).currentPSD;
    PSDAnalysisExport(realIndex).frequencyVector = app.PSDAnalysisResults(idx).frequencyVector;
    PSDAnalysisExport(realIndex).title = app.PSDAnalysisResults(idx).title;
    PSDAnalysisExport(realIndex).axesUnits = app.PSDAnalysisResults(idx).axesUnits;
  end
end
% Store temporary analysis struct in Data struct
Data.Analysis.PSD = PSDAnalysisExport;

% Define temporary analysis struct
DependenceAnalysisExport = struct([]);
realIndex = 0;
for idx = 1:length(app.DependenceAnalysisResults)
  % Don't store cleared Analysis
  if ~isempty(app.DependenceAnalysisResults(idx).xDep)
    % Avoid empty struct array rows (indexing with idx instead of
    % realIndex, if an analysis is cleared, makes that an empty struct
    % is stored)
    realIndex = realIndex+1;
    % Store analysis variables
    DependenceAnalysisExport(realIndex).xDep = app.DependenceAnalysisResults(idx).xDep;
    DependenceAnalysisExport(realIndex).yDep = app.DependenceAnalysisResults(idx).yDep;
    DependenceAnalysisExport(realIndex).title = app.DependenceAnalysisResults(idx).title;
    DependenceAnalysisExport(realIndex).axesUnits = app.DependenceAnalysisResults(idx).axesUnits;
  end
end
% Store temporary analysis struct in Data struct
Data.Analysis.Dependence = DependenceAnalysisExport;

% Save file
uisave({'Data'},nameOfFile)

% Display UIfigure over other windows
figure(app.EDAmatUIFigure)
end
