%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function GaussianFit(app)
% Fit every sweep
for sw = 1:app.fitVariableNumber
  
  % Check if data is a cell type (coming from selection of multiple cursor)
  if iscell(app.fitXCoords)
    x_fit = app.fitXCoords{sw};
    y_fit = app.fitYCoords{sw};
  else
    x_fit = app.fitXCoords(:, sw);
    y_fit = app.fitYCoords(:, sw);
  end
  
  % Check if last value is missing
  nanIdx = find(~isnan(y_fit), 1, 'last');
  x_fit = x_fit(1:nanIdx);
  y_fit = y_fit(1:nanIdx);
  
  % Initialize parameters
  A0 = sum(y_fit);
  mu0 = mean(x_fit);
  sigma0 = (max(x_fit)-min(x_fit))/5;
  
  %%%%% Fitting (least squares)
  % Initial conditions (defined above)
  x0 = [A0,mu0,sigma0];
  % fminsearch_error is the least square function; gauss_eval is the
  % function containing the model: modify this if you prefer to fit
  % another type of function
  fun = @(x)fminsearch_error(x,x_fit,y_fit,@gauss_eval);
  % Retrieve parameters
  gaussParam = fminsearch(fun,x0);
  
  % Save the fitted data in a matrix (to be plotted)...
  app.fitXResults(:, sw) = linspace(x_fit(1), x_fit(end), app.fitVariableLength);
  app.fitYResults(:, sw) = gauss_eval(gaussParam, app.fitXResults(:, sw));
  % ...and fitted parameters in a struct (to be exported)
  app.gaussParam(sw).GaussAmplitude =  gaussParam(1); % save parametrs in a struct
  app.gaussParam(sw).mean =  gaussParam(2);
  app.gaussParam(sw).std =  gaussParam(3);
  
end

% set parameter name and unit of measure in GUIFigure table
if strcmpi(app.tag, 'GUIFigure')
  app.UITable.ColumnName = {'Sweep';'mean [ms]';'std [ms]'};
elseif strcmpi(app.tag, 'AnalysisFigure') % to set parameter name and unit of measure in AnalysisAxes table
  % set parameter name and unit of measure in AnalysisAxes table
  app.UITable.ColumnName = {'Sweep';['mean ' app.xUnitLabel{end}];['std ' app.xUnitLabel{end}]};
end
% fill Table with parameters
app.UITable.Data = [ (1:length({app.gaussParam.mean}))' cell2mat({app.gaussParam.mean}') cell2mat({app.gaussParam.std}') ];
end
