%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function HistogramMenuSelected_m(app)
%%% Analysis: Histogram
InitialAnalysisControls(app);

% Execute Analysis only if cursors are in the right position
if app.cursorsOk
  % Dialog to choose the normalization to be applied to data
  d = dialog('Position',[300 200 250 150],'Name','I/V Data Selection','CloseRequestFcn',{@DataSelectionWindowClosing, app});
  
  txt1 = uicontrol('Parent',d,...
    'Style','text',...
    'Position',[0 90 250 50],...
    'String','Select the normalization');
  
  popupStringNorm = {'Not normalized','max [0 1]','Cell capacitance'};
  
  popupNorm = uicontrol('Parent',d,...
    'Style','popup',...
    'Position',[75 70 100 25],...
    'String',popupStringNorm,...
    'Callback',{@popupNorm_callback,app});
  
  btn = uicontrol('Parent',d,...
    'Style','pushbutton',...
    'Position',[89 20 70 25],...
    'String','Select',...
    'Callback',{@select_btn_callback,app});
  
  % Default: 1-> not normalized
  app.normType = app.NORM_TYPE_NOT;
  
  % Wait for d to close before running to completion
  uiwait(d);
  
  % Execute the fitting only if the dialog window hasn't been closed
  if ~app.analysisWindowClosed
    
    % Increment number of executed Analysis
    app.HISTindex = app.HISTindex+1;
    % Extract current data between cursors
    curSweep = app.current(app.cursorsTimeVect,:);
    
    % Define how many bins the histogram must have
    BinNumber = 25;
    % There are as many histogram as the number of sweeps
    HistNumber = size(curSweep, 2);
    % Allocate variables
    tempStruct.count = zeros(BinNumber, HistNumber);
    tempStruct.binCenters = zeros(BinNumber, HistNumber);
    
    % Plot on analysis figure
    f = AnalysisFigure;
    % Keep figure handle in a variable
    app.AnalysisFigureList = [app.AnalysisFigureList, f];
    % Retrieve axes handle
    a = f.getAxes;
    % Assign window closing function to AnalysisFigure
    a.DeleteFcn = {@AnalysisWindowClosing, app, a, f};
    % hold on
    set(a, 'nextplot', 'add');
    
    for sw = 1:HistNumber
      
      % Get number of points in every bin and bin edges
      [count,edges] = histcounts(curSweep(:, sw),BinNumber);
      % Calculate bin centers
      binCenters = (edges(1:end-1)+edges(2:end))/2;
      
      % Apply normalization
      if app.normType == 2
        count = count/ max(count);
      elseif app.normType == 3
        % Check if Cell capacitance value was inserted, otherwise use default
        % value = 1
        if isempty(app.cellCapacitance) || app.cellCapacitance == 0 || app.cellCapacitance == 1
          app.cellCapacitance = 1;
        end
        count = count/app.cellCapacitance;
      end
      
      % Plot
      h = bar(a,binCenters,count);
      % Histogram properties
      h.BarWidth = 1;
      % Thickness of bin edges
      h.EdgeAlpha = 0;
      % Transparency of bins
      h.FaceAlpha = 0.7;
      
      % Pass the original data to AnalysisAxes
      f.assignDataToPlot(binCenters',count',0);
      % Transpose vector to save them in the analysis struct
      tempStruct.count(:,sw) = count';
      tempStruct.binCenters(:,sw) = binCenters';
      
    end
    
    % Plot properties
    xLab = {'I' ['[' app.currentAxesLabel ']']};
    xlabel(a,xLab)
    % Select y label according to normalization selected
    switch app.normType
      case app.NORM_TYPE_NOT % not norm
        yLab = {'count'};
      case app.NORM_TYPE_MAX % [0 1]
        yLab = {'Fractional count'};
      case app.NORM_TYPE_CELL % Cell capacitance
        yLab = {'count ' '1/pF'};
    end
    ylabel(a,yLab)
    title(a,['Histogram ' num2str(app.HISTindex)])
    
    % Pass ticks data to AnalysisAxes
    f.savePlotTicks;
    % Pass labels to AnalysisAxes
    f.xUnitLabel = get(get(a,'xlabel'),'string');
    f.yUnitLabel = get(get(a,'ylabel'),'string');
    % Pass normalization type to AnalysisAxes
    f.normType = app.normType;
    % Save data in a struct to be exported
    tempStruct.axes = a;
    tempStruct.title = ['Histogram ' num2str(app.HISTindex)];
    tempStruct.axesUnits.Current = app.currentAxesLabel;
    tempStruct.axesUnits.Count = '[]';
    % Concatenate executed Analysis in a struct
    app.HistogramAnalysisResults = [app.HistogramAnalysisResults; tempStruct];
    
  end
  
end
end

function popupNorm_callback(popup,~,app)

% Set the normalization
idx = popup.Value;
popup_items = popup.String;
choice = string(popup_items{idx});
if startsWith(choice, "Not")
  app.normType = app.NORM_TYPE_NOT;
  
elseif startsWith(choice, "max")
  app.normType = app.NORM_TYPE_MAX;
  
elseif startsWith(choice, "Cell")
  app.normType = app.NORM_TYPE_CELL;
end

end

function select_btn_callback(~,~,~)

% Close dialog
delete(gcf)

end
