%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function LinearFit(app)
% Fit every sweep
for sw = 1:app.fitVariableNumber
  
  % Check if data is a cell type (coming from selection of multiple cursor)
  if iscell(app.fitXCoords)
    x_fit = app.fitXCoords{sw};
    y_fit = app.fitYCoords{sw};
  else
    x_fit = app.fitXCoords(:, sw);
    y_fit = app.fitYCoords(:, sw);
  end
  
  % Check if last value is missing
  nanIdx = find(~isnan(y_fit), 1, 'last');
  x_fit = x_fit(1:nanIdx);
  y_fit = y_fit(1:nanIdx);
  
  % Initialize parameters
  m0 = (y_fit(end)-y_fit(1))/(x_fit(end)-x_fit(1));
  b0 = median(y_fit);
  
  %%%%% Fitting (least squares)
  % Initial conditions (defined above)
  x0 = [m0,b0];
  % fminsearch_error is the least square function; linear_eval is the
  % function containing the model: modify this if you prefer to fit
  % another type of function
  fun = @(x)fminsearch_error(x,x_fit,y_fit,@linear_eval);
  % Retrieve parameters
  linearParam = fminsearch(fun,x0);
  
  % Save the fitted data in a matrix (to be plotted)...
  app.fitXResults(:, sw) = linspace(x_fit(1), x_fit(end), app.fitVariableLength);
  app.fitYResults(:, sw) = linear_eval(linearParam, app.fitXResults(:, sw));
  % ...and fitted parameters in a struct (to be exported)
  app.linParam(sw).Slope = linearParam(1);
  app.linParam(sw).yIntercept = linearParam(2);
  
end

% set parameter name and unit of measure in GUIFigure table
if strcmpi(app.tag, 'GUIFigure')
  app.UITable.ColumnName = {'Sweep'; 'Slope [nA/ms]'; 'Intercept [nA]'};
elseif strcmpi(app.tag, 'AnalysisFigure')  % to set parameter name and unit of measure in AnalysisAxes table
  % set parameter name and unit of measure in AnalysisAxes table
  switch app.normType
    % Unit of measures depends on normalization
    case app.NORM_TYPE_NOT
      app.UITable.ColumnName = {'Sweep'; ['Slope ' app.yUnitLabel{end} '/' app.xUnitLabel{end} ]; [ 'Intercept ' app.yUnitLabel{end}]};
    case app.NORM_TYPE_MAX
      app.UITable.ColumnName = {'Sweep'; ['Slope 1/' app.xUnitLabel{end} ]; 'Intercept '};
    case app.NORM_TYPE_CELL
      app.UITable.ColumnName = {'Sweep'; ['Slope ' app.yUnitLabel{end} ' /' app.xUnitLabel{end} ];[ 'Intercept ' app.yUnitLabel{end} ]};
  end
end
% fill Table with parameters
app.UITable.Data = [(1:length({app.linParam.Slope}))' cell2mat({app.linParam.Slope}') cell2mat({app.linParam.yIntercept}')];
end
