%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function LoadMatActions(app, Data)
% Transfer data from "Data" struct to variables used downstream
app.dataTimeVect               = Data.Time;
app.current                    = Data.Current;
app.voltage                    = Data.Voltage;
app.fileName                   = Data.FileName;
app.infoAbfLoad.uFileStartDate = Data.DateTime;
app.samplingTime               = (app.dataTimeVect(2)-app.dataTimeVect(1));
app.samplingFrequency          = Data.SamplingFrequency;
app.sweepLength                = size(app.current, 1);
app.sweepsNumber               = size(app.voltage, 2);
app.channelsNumber             = size(app.current, 2)/app.sweepsNumber;

%%% Converting to used units of measure
% Current     -> [nA]
% Voltage     -> [mV]
% Time        -> [ms]
% Frequency   -> [kHz]
% Conductance -> [uS]

% Check if current is in nA, else convert it to nA
if strcmpi(Data.AxesUnits.Current,'nA')
  % nothing to do
elseif stracmpi(Data.AxesUnits.Current,'pA')
  app.current = app.current/1e3;
end

% Check if voltage is in mV, else convert it to mV
if strcmpi(Data.AxesUnits.Voltage,'mV')
  % nothing to do
elseif stracmpi(Data.AxesUnits.Voltage,'uV')
  app.voltage = app.voltage/1e3;
end

% Check if time is in ms, else convert it to ms
if strcmpi(Data.AxesUnits.Time,'ms')
  % nothing to do
elseif stracmpi(Data.AxesUnits.Time,'us')
  app.samplingTime = app.samplingTime/1e3;
end

% Check if frequency is in kHz, else convert it to kHz
if strcmpi(Data.AxesUnits.SamplingFrequency,'kHz')
  % nothing to do
elseif strcmpi(Data.AxesUnits.SamplingFrequency,'Hz')
  app.samplingFrequency = app.samplingFrequency/1e-3;
end

% Define unit labels
app.currentAxesLabel = 'nA';
app.voltageAxesLabel = 'mV';
app.timeAxesLabel = 'ms';
app.freqAxesLAbel = 'kHz';
% conductance depends on I and V: [nA]/[mV] = [uS]
app.conductanceAxesLabel = 'uS';

%%% Loading exported analysis
if ~isempty(Data.Analysis.IVGraph)
  app.IVAnalysisResults = Data.Analysis.IVGraph;
end
if ~isempty(Data.Analysis.ConductanceGraph)
  app.ConductanceAnalysisResults = Data.Analysis.ConductanceGraph;
end
if ~isempty(Data.Analysis.Histogram)
  app.HistogramAnalysisResults = Data.Analysis.Histogram;
end
if ~isempty(Data.Analysis.PSD)
  app.PSDAnalysisResults = Data.Analysis.PSD;
end

% Transfer parameters from "Data" struct to variables used downstream
app.analysisFittedParameters = Data.AnalysisFittedParameters;
app.expParam = Data.FittedParameters.Exponential;
app.gaussParam = Data.FittedParameters.Gaussian;
app.linParam = Data.FittedParameters.Linear;
app.boltzParam = Data.FittedParameters.Boltzmann;

% hold on
set(app.CurrentAxes,'nextplot','add')

% Plot current data
plot(app.CurrentAxes,app.dataTimeVect, app.current)
title(app.CurrentAxes,'Current')
xlabel(app.CurrentAxes,'ms')
ylabel(app.CurrentAxes, app.currentAxesLabel)

% Plot voltage data
plot(app.VoltageAxes,app.dataTimeVect, app.voltage)
title(app.VoltageAxes,'Voltage')
xlabel(app.VoltageAxes,'ms')
ylabel(app.VoltageAxes, app.voltageAxesLabel)

% Enable grids
grid(app.CurrentAxes, 'on')
grid(app.VoltageAxes, 'on')

% Enable fields and spinners
app.CursorSpinnerLabel.Enable = 'on';
app.CursorNumberSpinner.Enable = 'on';
app.DataInfoMenu.Enable = 'on';
app.ExporttomatMenu.Enable = 'on';
app.RestingPotentialField.Enable = 'on';
app.RestingPotentialmVLabel.Enable = 'on';

% Enable analysis and fittings
app.IVgraphMenu.Enable = 'on';
app.HistogramMenu.Enable = 'on';
app.ConductancegraphMenu.Enable = 'on';
app.PSDMenu.Enable = 'on';
app.DependenceAnalysisMenu.Enable = 'on';
app.ExponentialMenu.Enable = 'on';
app.GaussianMenu.Enable = 'on';
app.LinearMenu.Enable = 'on';
app.BoltzmannMenu.Enable = 'on';
app.ExperimentalconditionField.Enable = 'on';
app.ExperimentalconditionLabel.Enable = 'on';
app.CellCapacitancepFLabel.Enable = 'on';
app.CellCapacitanceField.Enable = 'on';
app.UITable.Enable = 'on';

% Display UIfigure over other windows
figure(app.EDAmatUIFigure)
end
