%   Copyright (C) 2019 Eugenio Ricci
%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function PSDMenuSelected_m(app)
%%% Analysis: Power Spectral Density using FFT (Welch's method)
InitialAnalysisControls(app);

% Execute Analysis only if cursors are in the right position
if app.cursorsOk
  % Dialog to choose the normalization to be applied to data
  d = dialog('Position',[300 200 250 150],'Name','I/V Data Selection','CloseRequestFcn',{@DataSelectionWindowClosing, app});
  
  txt1 = uicontrol('Parent',d,...
    'Style','text',...
    'Position',[0 90 250 50],...
    'String','Select the normalization');
  
  popupStringNorm = {'Not normalized','max [0 1]','Cell capacitance'};
  
  popupNorm = uicontrol('Parent',d,...
    'Style','popup',...
    'Position',[75 70 100 25],...
    'String',popupStringNorm,...
    'Callback',{@popupNorm_callback,app});
  
  btn = uicontrol('Parent',d,...
    'Style','pushbutton',...
    'Position',[89 20 70 25],...
    'String','Select',...
    'Callback',{@select_btn_callback,app});
  
  % Default: 1-> not normalized
  app.normType = 1;
  
  % Wait for d to close before running to completion
  uiwait(d);
  
  % Execute the fitting only if the dialog window hasn't been closed
  if ~app.analysisWindowClosed
    
    % Increment number of executed Analysis
    app.PSDindex = app.PSDindex+1;
    % Get current and voltage data between cursors
    curSweep = app.current(app.cursorsTimeVect,:);
    voltSweep = app.voltage(app.cursorsTimeVect,:);
    
    
    L = length(curSweep);
    Fs = app.samplingFrequency;
    % Number of (50% overlapping) windows
    numWin = 8;
    % Number of non-overlapping windows inside the signal
    lengthFactor = 4.5;
    % Number of samples per window
    lengthWin = floor(L/lengthFactor);
    % 50% overlap
    numOvLap = floor(lengthWin*0.5);
    % NUmber of transform points
    nfft = max(256,2^nextpow2(lengthWin));
    
    % Indices at which every window starts
    startWin = 1:lengthWin;
    
    % Hamming Window
    N = lengthWin-1;
    n = 0:N;
    hammWin = (0.54-0.46*cos(2*pi*n/N))/0.54;
    
    % Initialization
    Win = zeros(numWin,lengthWin,size(app.current,2));
    winTrasf = zeros(numWin,nfft,size(app.current,2));
    
    % Transform every sweep
    for sw = 1:size(app.current,2)
      % Extract and transform every (hanning) window
      for k = 1:numWin
        Win(k,:,sw) = curSweep(startWin+(k-1)*numOvLap,sw); % (1:L)+(K-1)*D -> Welch, 1967
        Win(k,:,sw) = (Win(k,:,sw)).*hammWin;
        winTrasf(k,:,sw) = fft(Win(k,:,sw),nfft);
      end
    end
    
    % Extracting one side of the fft
    winTrasf = winTrasf(:,1:(length(winTrasf)/2+1),:);
    % Averaging windows spectra
    app.psdMod = squeeze(mean( (1/lengthWin/Fs) * abs(winTrasf).^2));
    % Defining frequency axis
    psdFreq = (Fs/nfft:Fs/nfft:Fs/2)';
    
    % Delete DC component (first sample)
    if ~(size(curSweep,2)>1)
      app.psdMod = reshape(app.psdMod,length(app.psdMod),size(curSweep,2));
    end
    app.psdMod = 2*app.psdMod(2:end,:);
    
    % Apply normalization and define y label
    switch app.normType
      case app.NORM_TYPE_NOT % not norm
        yLab = { 'Power/Frequency' ['[nA^{2}/' app.freqAxesLAbel ']' ]};
      case app.NORM_TYPE_MAX % [0 1]
        app.psdMod = app.psdMod./max(app.psdMod(:,1));
        yLab = {'Fractional Activation'};
      case app.NORM_TYPE_CELL % Cell capacitance
        % Check if Cell capacitance value was inserted, otherwise use default
        % value = 1
        if isempty(app.cellCapacitance) || app.cellCapacitance == 0 || app.cellCapacitance == 1
          app.cellCapacitance = 1;
        end
        app.psdMod = app.psdMod/app.cellCapacitance;
        yLab = {'Power/Frequency' ['[nA^{2}/' app.freqAxesLAbel '*(1/pF)]' ]};
    end
    
    % Plot on analysis figure
    f = AnalysisFigure;
    % Keep figure handle in a variable
    app.AnalysisFigureList = [app.AnalysisFigureList, f];
    % Retrieve axes handle
    a = f.getAxes;
    % Assign window closing function to AnalysisFigure
    a.DeleteFcn = {@AnalysisWindowClosing, app, a, f};
    % hold on
    set(a, 'nextplot', 'add');
    
    x = psdFreq;
    for plotIdx = 1:size(app.psdMod, 2)
      y = app.psdMod(:, plotIdx);
      plot(a, x, y)
      %Pass the original data to AnalysisAxes
      f.assignDataToPlot(x, y,0);
    end
    
    % Plot properties
    grid(a, 'on')
    title(a,['PSD ' num2str(app.PSDindex)])
    xLab = {'Frequency ' ['[' app.freqAxesLAbel ']']};
    xlabel(a, xLab)
    ylabel(a,yLab)
    
    % Pass the ticks data to AnalysisAxes
    f.savePlotTicks;
    f.xUnitLabel = xLab;
    f.yUnitLabel = yLab;
    % Pass normalization type to AnalysisAxes
    f.normType = app.normType;
    % Save data in a struct
    tempStruct.currentPSD = app.psdMod;
    tempStruct.frequencyVector = repmat(psdFreq,1,10);
    tempStruct.axes = a;
    tempStruct.title = ['PSD ' num2str(app.PSDindex)];
    tempStruct.axesUnits.Frequency = 'kHz';
    tempStruct.axesUnits.PowerDensity  = 'nA^{2}/kHz';
    % Concatenate executed Analysis in a struct
    app.PSDAnalysisResults = [app.PSDAnalysisResults; tempStruct];
    
  end
  
end
end

function popupNorm_callback(popup,~,app)

% Set the normalization
idx = popup.Value;
popup_items = popup.String;
choice = string(popup_items{idx});
if startsWith(choice, "Not")
  app.normType = app.NORM_TYPE_NOT;
  
elseif startsWith(choice, "max")
  app.normType = app.NORM_TYPE_MAX;
  
elseif startsWith(choice, "Cell")
  app.normType = app.NORM_TYPE_CELL;
end

end

function select_btn_callback(~,~,~)

% Close dialog
delete(gcf)

end
