%   Copyright (C) 2020 Filippo Cona
%
%   This file is part of EDAmat.
%
%   EDAmat is free software: you can redistribute it and/or modify
%   it under the terms of the GNU Lesser General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   EDAmat is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public License
%   along with EDAmat.  If not, see <http://www.gnu.org/licenses/>.

function [current, voltage, time, dataInfo] = loadEdh(varargin)
%loadEdh   Read data saved in binary format or .abf by EDR3, or data
%   exported in binary format, .csv or .abf by EDA
%   current = loadEdh() opens a file browser to select an elements
%   data header to open. current is matrix with one column for each current
%   channel.
%
%   [current, voltage, time] = loadEdh() returns also a column vector
%   voltage containing the voltage command data and a column vector time
%   containting the time array
%
%   [..., dataInfo] = loadEdh() returns also a structure dataInfo that
%   contains general information on the data loaded:
%
%   headerVersion       - version of the edh file
%   currentChannelsNum  - number of current channels of the device
%   currentRangeValue   - numerical value of the current range
%   currentRangeUnit    - unit of the current range
%   samplingRateValue   - numerical value of the sampling rate
%   samplingRateUnit    - unit of the sampling rate
%   maxBandwidth        - maximum bandwidth of the recorded signal
%   activeChannelsNum   - number of active current channels
%   activeChannels      - indexes of the active current channels
%   channelSamplesNum   - number of samples per channel
%
%   loadEdh() with no ouputs plots the loaded data
%
%   loadEdh(fileFullPath) takes the full path fileFullPath to the .edh
%   file to be loaded
%
%   loadEdh(pathName, headerFileName) takes the path pathName and the
%   file name headerFileName of the header file

%% definitions
NONE = 0;

DEV_SR_1_25_KHZ = 1.25e6;
DEV_SR_5_KHZ = 1.25e6;
DEV_SR_10_KHZ = 1.25e6;
DEV_SR_20_KHZ = 1.25e6;
DEV_SR_50_KHZ = 10e6;
DEV_SR_100_KHZ = 10e6;
DEV_SR_200_KHZ = 10e6;

DEV_OSR_1_25_KHZ = 1024;
DEV_OSR_5_KHZ = 256;
DEV_OSR_10_KHZ = 128;
DEV_OSR_20_KHZ = 64;
DEV_OSR_50_KHZ = 200;
DEV_OSR_100_KHZ = 100;
DEV_OSR_200_KHZ = 50;

SR_2 = 1/2;
SR_8 = 1/8;
SR_10 = 1/10;
SR_20 = 1/20;

DAT = 1;
ABF = 2;
CSV = 3;

%% parse the arguments
if nargin == 0
  % No header file provided, select one
  [headerFileName, pathName] = uigetfile("*.edh");
  
elseif nargin == 1
  % One string provided, must be the absolute or relative path to the file
  % extract path part
  folders = regexp(string(varargin{1}), filesep, 'split');
  if folders == ""
    error("Error: invalid file path")
    
  elseif length(folders) == 1
    pathName = ".";
    
  else
    pathName = folders(1);
    for idx = 2:length(folders)-1
      pathName = fullfile(pathName, folders(idx));
    end
  end
  headerFileName = folders(end);
  
elseif nargin == 2
  % Two strings provided, must be the absolute or relative path name and
  % file name
  pathName = varargin{1};
  headerFileName = varargin{2};
end

% check for valid data file name
if isnumeric(headerFileName)
  disp("Error: invalid file name");
  return
end

if ~endsWith(headerFileName, ".edh")
  disp("Error: invalid file name (.edh required)");
  return
end

if strlength(headerFileName) < 5
  disp("Error: invalid file name");
  return
end

% get .edh version
headerFid = fopen(fullfile(pathName, headerFileName));
headerLine = string(fgetl(headerFid));
if startsWith(headerLine, "EDH Version:")
  if endsWith(headerLine, " 2.0")
    dataInfo.headerVersion = "2.0";
  else
    dataInfo.headerVersion = "1.0";
  end
else
  dataInfo.headerVersion = "1.0";
  fseek(headerFid, 0, "bof"); % if the header file does not contain the line "EDH Version:"
  % do not discard the first line
end

% default values
dataInfo.currentChannelsNum = 1;
dataInfo.currentRangeValue = 200;
dataInfo.currentRangeUnit = "pA";
deviceSr = DEV_SR_1_25_KHZ; % default 1.25MHz sampling before decimation
deviceOSR = DEV_OSR_1_25_KHZ; % default decimation to get ~1.25 kHz
dataInfo.samplingRateValue = DEV_SR_1_25_KHZ/DEV_OSR_1_25_KHZ;
dataInfo.samplingRateUnit = "Hz";
finalBandWidthCoeff = SR_2; % Post Filter cut off frequency. Default = no filter
oversamplingCoeff = 1; % oversampling applied to original sampling rate
dataInfo.activeChannelsNum = 1;
dataInfo.activeChannels = 1;

%% parse the header
while true
  headerLine = fgetl(headerFid);
  if ~ischar(headerLine)
    break % eof
  end
  
  headerLine = string(headerLine);
  headerLine = strtrim(headerLine); % remove leading and trailing whitespaces
  headerLine = regexprep(headerLine, " +", " "); % remove multiple whitespaces
  
  if headerLine == ""
    % skip empty lines
    continue
  end
  
  words = split(headerLine, " ");
  
  % Channels line e.g. Channels: 1
  if startsWith(words(1), "channels", "IgnoreCase", true)
    dataInfo.currentChannelsNum = str2double(words(2));
  end
  
  % Range line e.g. Range: 200 pA
  if startsWith(words(1), "range", "IgnoreCase", true)
    dataInfo.currentRangeValue = str2double(words(2));
    dataInfo.currentRangeUnit = words(3);
  end
  
  % Sampling rate line e.g. Sampling frequency (SR): 1.25 kHz
  if (startsWith(words(1), "sampling", "IgnoreCase", true) && ...
      startsWith(words(2), "frequency", "IgnoreCase", true) && ...
      startsWith(words(3), "(sr)", "IgnoreCase", true))
    switch words(4)
      case "1.25"
        deviceOSR = DEV_OSR_1_25_KHZ;
        deviceSr = DEV_SR_1_25_KHZ;
        
      case "5"
        deviceOSR = DEV_OSR_5_KHZ;
        deviceSr = DEV_SR_5_KHZ;
        
      case "10"
        deviceOSR = DEV_OSR_10_KHZ;
        deviceSr = DEV_SR_10_KHZ;
        
      case "20"
        deviceOSR = DEV_OSR_20_KHZ;
        deviceSr = DEV_SR_20_KHZ;
        
      case "50"
        deviceOSR = DEV_OSR_50_KHZ;
        deviceSr = DEV_SR_50_KHZ;
        
      case "100"
        deviceOSR = DEV_OSR_100_KHZ;
        deviceSr = DEV_SR_100_KHZ;
        
      case "200"
        deviceOSR = DEV_OSR_200_KHZ;
        deviceSr = DEV_SR_200_KHZ;
    end
  end
  
  % Bandwidth line e.g. Final Bandwidth: SR/2 (no filter)
  if (startsWith(words(1), "final", "IgnoreCase", true) && ...
      startsWith(words(2), "bandwidth", "IgnoreCase", true))
    switch lower(words(3))
      case "sr/2"
        finalBandWidthCoeff = SR_2;
        
      case "sr/8"
        finalBandWidthCoeff = SR_8;
        
      case "sr/10"
        finalBandWidthCoeff = SR_10;
        
      case "sr/20"
        finalBandWidthCoeff = SR_20;
    end
  end
  
  % Oversampling line e.g. Oversampling x4: enabled
  if startsWith(words(1), "oversampling", "IgnoreCase", true)
    if strcmpi(words(3), "enabled")
      oversamplingCoeff = str2double(extractBetween(words(2), 2, strlength(words(2))-1));
      
    else
      oversamplingCoeff = 1;
    end
  end
  
  % Active channels line e.g. Active channels: 1 3 4
  if (startsWith(words(1), "active", "IgnoreCase", true) && ...
      startsWith(words(2), "channels", "IgnoreCase", true))
    dataInfo.activeChannelsNum = length(words)-2;
    dataInfo.activeChannels = zeros(dataInfo.activeChannelsNum, 1);
    for idx = 1:dataInfo.activeChannelsNum
      dataInfo.activeChannels(idx) = str2double(words(idx+2));
    end
  end
end

dataInfo.samplingRateValue = deviceSr/deviceOSR*oversamplingCoeff;
dataInfo.maxBandwidth = deviceSr/deviceOSR*finalBandWidthCoeff;

fclose(headerFid);

%% load the data
% get data file type (.dat and .abf supported)
dataFileType = NONE;
fileList = dir(pathName);
for fileIdx = 1:length(fileList)
  if endsWith(fileList(fileIdx).name, ".dat")
    dataFileType = DAT;
    dataFileExtension = ".dat";
    
  elseif endsWith(fileList(fileIdx).name, ".abf")
    dataFileType = ABF;
    dataFileExtension = ".abf";
    
  elseif endsWith(fileList(fileIdx).name, ".csv")
    dataFileType = CSV;
    dataFileExtension = ".csv";
  end
end

allDataFiles = [];
for fileIdx = 1:length(fileList)
  if endsWith(fileList(fileIdx).name, dataFileExtension)
    allDataFiles = [allDataFiles, fileList(fileIdx)];
  end
end
dataFilesNum = length(allDataFiles);
 
switch dataFileType
  case DAT
    totalSamplesNum = 0;
    for fileIdx = 1:dataFilesNum
      totalSamplesNum = totalSamplesNum+allDataFiles(fileIdx).bytes/4;
    end
    
    totalData = zeros(totalSamplesNum, 1);
    offset = 0;
    for fileIdx = 1:dataFilesNum
      dataFid = fopen(fullfile(pathName, allDataFiles(fileIdx).name), "rb");
      fileData = fread(dataFid, 'single');
      totalData(offset+(1:length(fileData))) = fileData;
      offset = offset+length(fileData);
      fclose(dataFid);
    end
    dataInfo.channelSamplesNum = totalSamplesNum/(1+dataInfo.activeChannelsNum);
    totalData = reshape(totalData, 1+dataInfo.activeChannelsNum, dataInfo.channelSamplesNum)';
    
  case ABF
    fileIdx = 1;
    channelIdx = 1;
    while fileIdx <= dataFilesNum
      channelStr = extractBetween(string(allDataFiles(fileIdx).name), ...
        strlength(allDataFiles(fileIdx).name)-10, ...
        strlength(allDataFiles(fileIdx).name)-8);
      [channelData, ~, ~, e] = abfload(char(fullfile(pathName, allDataFiles(fileIdx).name)), "doDispInfo", false);
      if e == -1
        channelData = [];
      end
      
      fileIdx = fileIdx+1;
      while (fileIdx <= dataFilesNum && ...
          channelStr == extractBetween(string(allDataFiles(fileIdx).name), ...
          strlength(allDataFiles(fileIdx).name)-10, ...
          strlength(allDataFiles(fileIdx).name)-8))
        
        [newChannelData, ~, ~, e] = abfload(char(fullfile(pathName, allDataFiles(fileIdx).name)), "doDispInfo", false);
        if e == -1
          newChannelData = [];
        end
        channelData = [channelData; newChannelData];
        fileIdx = fileIdx+1;
      end
      
      if channelIdx == 1
        dataInfo.channelSamplesNum = size(channelData, 1);
        totalData = zeros(dataInfo.channelSamplesNum, dataInfo.activeChannelsNum+1);
        totalData(:, channelIdx) = channelData(:, 1);
        totalData(:, end) = channelData(:, 2);
        
      else
        totalData(:, channelIdx) = channelData(:, 1);
      end
      channelIdx = channelIdx+1;
    end
    
  case CSV
    if dataFilesNum ~= 1
      error('load of only single .csv files supported')
    end
    
    opts = delimitedTextImportOptions("NumVariables", 1+dataInfo.activeChannelsNum);
    opts.DataLines = [1, Inf];
    opts.Delimiter = ",";
    opts.VariableTypes = repmat("double", 1, 1+dataInfo.activeChannelsNum);
    opts.ExtraColumnsRule = "ignore";
    opts.EmptyLineRule = "read";
    totalData = table2array(readtable(fullfile(pathName, allDataFiles(1).name), opts));
    clear opts

    if isnan(totalData(1, 1))
      totalData(1, :) = [];
    end
    dataInfo.channelSamplesNum = size(totalData, 1);
    
  case NONE
    error("No valid data files found")
end

current = totalData(:, 1:dataInfo.activeChannelsNum);
voltage = totalData(:, end);
time = ((1:dataInfo.channelSamplesNum)/dataInfo.samplingRateValue)';

%% plot data
if nargout == 0
  plotHandles = zeros(dataInfo.activeChannelsNum+1, 1);
  
  figure
  plotHandles(1) = subplot(2, 1, 1);
  ColOrd = get(gca, "ColorOrder");
  m = size(ColOrd, 1);
  for i = 1:dataInfo.activeChannelsNum
    ColRow = rem(i-1, m)+1;
    Col = ColOrd(ColRow, :);
    plot(time, current(:, i), "Color", Col)
    hold on
  end
  xlim(time([1, end]))
  grid on
  xlabel("Time [s]")
  ylabel("I [" + dataInfo.currentRangeUnit + "]")
  
  plotHandles(2) = subplot(2, 1, 2);
  plot(time, voltage)
  xlim(time([1, end]))
  grid on
  xlabel("Time [s]")
  ylabel("Vc [mV]")
  linkaxes(plotHandles, "x")
end
end
